/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is wmlbrowser.
 *
 * The Initial Developer of the Original Code is
 * Matthew Wilson <matthew@mjwilson.demon.co.uk>.
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/**
 * A WebProgressListener for the wmlbrowser extension.
 * @param b Browser to monitor
 * @param d Direction of web progress (1=forwards or new entry, -1=back.
 */
function wmlbrowserWebProgressListener (b,d) {
    this.browser = b;
    this.direction = d;
    this.logger =
        Components.classes['@mozilla.org/consoleservice;1'].getService(Components.interfaces.nsIConsoleService);
    this.rnd = Math.random();
    this.logger.logStringMessage ("Created web progress listener, direction="+this.direction + ", rnd="+this.rnd);
}

/**
 * The namespace used for our internal attributes set on the
 * transformed HTML document.
 */
wmlbrowserWebProgressListener.prototype.WMLBROWSER_NS = "http://wmlbrowser.mozdev.org/ns/forms#";

wmlbrowserWebProgressListener.prototype.log = function(message) {
    this.logger.logStringMessage (message);
};

/**
 * Standard QueryInterface function.
 * This module implements nsIWebProgressListener, nsISupportsWeakReference,
 * and nsISupports.
 */
wmlbrowserWebProgressListener.prototype.QueryInterface = function(aIID)
{
    if (aIID.equals(Components.interfaces.nsIWebProgressListener) ||
        aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
        aIID.equals(Components.interfaces.nsISupports))
        return this;
    throw Components.results.NS_NOINTERFACE;
};

wmlbrowserWebProgressListener.prototype.afterLoad = function (aProgress, aURI) {
    try {
        this.log ("wmlbrowserWebProgressListener.afterLoad direction="+this.direction + ", rnd=" + this.rnd + ", uri="+aURI);

        var doc = aProgress.DOMWindow.document;

        if (doc.documentElement.hasAttributeNS (this.WMLBROWSER_NS, "wmlbrowser")) {
            var card;
            if (aURI.indexOf ("#") > 0) {
                var fragid = aURI.substring (1+aURI.indexOf("#"));
                var card = doc.getElementById (fragid);
            } else {
                // First card. (NB first div is in the 'noscript' section.)
                card = doc.getElementsByTagNameNS ("http://www.w3.org/1999/xhtml","div")[1];
            }
            if (card) {

                this.log ("wmlbrowserWebProgressListener.afterLoad dispatching event");

                var e = doc.createEvent ("Event");
                e.initEvent ("onformentered", true, true);
                card.dispatchEvent (e);

                if (this.direction == +1) {
                    var e = doc.createEvent ("Event");
                    e.initEvent ("onenterforward", true, true);
                    card.dispatchEvent (e);
                } else if (this.direction == -1) {
                    var e = doc.createEvent ("Event");
                    e.initEvent ("onenterbackward", true, true);
                    card.dispatchEvent (e);
                }
            } else {
                this.log ("wmlbrowserWebProgressListener.afterLoad discarding, no card found");
            }
        } else {
            this.log ("wmlbrowserWebProgressListener.afterLoad discarding non-WML document");
        }
    } catch (e) {
        this.log (e);
        Components.utils.reportError (e);
    }
}

wmlbrowserWebProgressListener.prototype.onStateChange = function(aProgress,aRequest,aFlag,aStatus)
{
    //this.log ("wmlbrowserWebProgressListener.onStateChange flags=" + aFlag);
    if (aFlag & Components.interfaces.nsIWebProgressListener.STATE_STOP) {
        this.log ("wmlbrowserWebProgressListener.onStateChange STATE_STOP, calling afterLoad");

        //this.afterLoad (aProgress, aRequest.name);
        var pl = this;
        window.setTimeout ( function () { pl.afterLoad (aProgress, aRequest.name); } , 100); // TODO wait for some kind of DOMLoaded event?

        this.log ("removing progressListener with direction " + this.direction + ", rnd=" + this.rnd);
        this.browser.removeProgressListener (this);
    }
};

wmlbrowserWebProgressListener.prototype.onLocationChange = function(aProgress,aRequest,aLocation) {
    this.log ("wmlbrowserWebProgressListener.onLocationChange  location=" + aLocation.spec + ", document=" + aProgress.DOMWindow.document + ", isLoadingDocument=" + aProgress.isLoadingDocument + ", title="+aProgress.DOMWindow.document.title + ", rnd=" + this.rnd);
    if (!aProgress.isLoadingDocument) {
        this.log ("wmlbrowserWebProgressListener.onLocationChange rnd=" + this.rnd + ", calling afterLoad");

        //this.afterLoad (aProgress, aLocation.spec);
        var pl = this;
        window.setTimeout ( function () { pl.afterLoad (aProgress, aLocation.spec); } , 0);

        this.log ("removing progressListener with direction " + this.direction + ", rnd=" + this.rnd);
        this.browser.removeProgressListener (this);
    }
};

wmlbrowserWebProgressListener.prototype.onProgressChange =
  function(aWebProgress,aRequest,aCurSelfProgress,aMaxSelfProgress,aCurTotalProgress,aMaxTotalProgress) {
      this.log ("wmlbrowserWebProgressListener.onProgressChange " + aCurSelfProgress + "/" + aMaxSelfProgress);
};
wmlbrowserWebProgressListener.prototype.onStatusChange = function(a,b,c,d){
    this.log ("wmlbrowserWebProgressListener.onStatusChange");
};
wmlbrowserWebProgressListener.prototype.onSecurityChange =
   function(aWebProgress,aRequest,aState) {
}; 


function wmlbrowserHistoryListener (b) {
    this.browser = b;
    this.logger =
        Components.classes['@mozilla.org/consoleservice;1'].getService(Components.interfaces.nsIConsoleService);
}

wmlbrowserHistoryListener.prototype.log = function(message) {
    this.logger.logStringMessage (message);
};

wmlbrowserHistoryListener.prototype.QueryInterface = function(aIID)
{
    if (aIID.equals(Components.interfaces.nsISHistoryListener) ||
        aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
        aIID.equals(Components.interfaces.nsISupports))
        return this;
    throw Components.results.NS_NOINTERFACE;
}; 

wmlbrowserHistoryListener.prototype.OnHistoryNewEntry = function (uri) {
    this.log ("OnHistoryNewEntry uri=" + uri.spec);
    try {
        this.browser.addProgressListener
               (new wmlbrowserWebProgressListener(this.browser, 1),
                Components.interfaces.nsIWebProgress.NOTIFY_STATE_REQUEST ||
                Components.interfaces.nsIWebProgress.STATE_IS_DOCUMENT ||
                Components.interfaces.nsIWebProgress.STATE_IS_NETWORK ||
                Components.interfaces.nsIWebProgress.STATE_IS_WINDOW
                );
    } catch (e) {
        this.log (e);
        Components.utils.reportError (e);
    }
    return true;
};

wmlbrowserHistoryListener.prototype.OnHistoryGoBack = function (uri) {
    this.log ("OnHistoryGoBack " + uri.spec);
    try {
        this.browser.addProgressListener
               (new wmlbrowserWebProgressListener(this.browser, -1),
                Components.interfaces.nsIWebProgress.NOTIFY_STATE_REQUEST ||
                Components.interfaces.nsIWebProgress.STATE_IS_DOCUMENT ||
                Components.interfaces.nsIWebProgress.STATE_IS_NETWORK ||
                Components.interfaces.nsIWebProgress.STATE_IS_WINDOW
                );
    } catch (e) {
        this.log (e);
        Components.utils.reportError (e);
    }
    return true;
};

wmlbrowserHistoryListener.prototype.OnHistoryGoForward = function (uri) {
    this.log ("OnHistoryGoForward " + uri.spec);
    try {
        this.browser.addProgressListener
               (new wmlbrowserWebProgressListener(this.browser, 1),
                Components.interfaces.nsIWebProgress.NOTIFY_STATE_REQUEST ||
                Components.interfaces.nsIWebProgress.STATE_IS_DOCUMENT ||
                Components.interfaces.nsIWebProgress.STATE_IS_NETWORK ||
                Components.interfaces.nsIWebProgress.STATE_IS_WINDOW
                );
    } catch (e) {
        this.log (e);
        Components.utils.reportError (e);
    }
    return true;
};

wmlbrowserHistoryListener.prototype.OnHistoryReload = function (uri) {
    this.log ("OnHistoryReload " + uri.spec);
    return true;
};

wmlbrowserHistoryListener.prototype.OnHistoryPurge = function (uri) {
    this.log ("OnHistoryPurge " + uri.spec);
    return true;
}


wmlbrowserHistoryListener.prototype.OnHistoryGotoIndex = function (index, uri) {
    this.log ("OnHistoryGotoIndex " + uri.spec);
    return true;
}


// http://forums.mozillazine.org/viewtopic.php?p=2381777&sid=61f73c3c12953f5f195deb9b18eb3030
function initialiseWmlbrowserHistoryListener() {

    try {

        // add first History Listener to focused browser at start-up.
        gBrowser.wmlbrowserHistoryListener = new wmlbrowserHistoryListener(gBrowser);
        gBrowser.sessionHistory.addSHistoryListener(gBrowser.wmlbrowserHistoryListener);

        // listen for new tabs and add a HistoryListener
        gBrowser.tabContainer.addEventListener("TabOpen",

            function(evt) {

                if(evt.target.localName == "tab") {

                    var tab = evt.target;

/*
                    // wait until new Browser is initialized
                    setTimeout(
                        function() {
*/
                            try {

                                var browser = gBrowser.getBrowserForTab(tab);

                                browser.wmlbrowserHistoryListener = new wmlbrowserHistoryListener(browser);
                                browser.sessionHistory.addSHistoryListener( browser.wmlbrowserHistoryListener );

                            } catch (e) {
                                Components.classes['@mozilla.org/consoleservice;1']
                                    .getService(Components.interfaces.nsIConsoleService)
                                    .logStringMessage (e);
                            }
/*
                        },
                        50);
*/
                }
            }
            , false);

    } catch (e) {
         Components.classes['@mozilla.org/consoleservice;1']
            .getService(Components.interfaces.nsIConsoleService)
               .logStringMessage (e);
         Components.utils.reportError (e);
    }
};

window.addEventListener("load", function() { initialiseWmlbrowserHistoryListener(); }, false); 
